/*
 * Copyright (C) 1996-2003 LAAS/CNRS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "config.h"
__RCSID("$LAAS$");

/******************************************************\
**                    client.c                        **
**                   ----------                       **
**  This file implements all functions useful for the **
** user. The user should always call these functions, **
** and not bother with the packets themselves.        **
**                                                    **
**  The included functions are:                       **
**                                                    **
**     int get_connection ( char* )                   **
**     int place_robot ( int, position* )             **
**     int erase_robot ( int )                        **
**     int set_platform ( int, angle* )               **
**     int place_diamond ( v_position* )              **
**     int erase_diamond ()                           **
**     int place_doors ( v_segment* )                 **
**     int erase_doors ()                             **
**     int place_walls ( v_segment* )                 **
**     int place_walls_rel ( v_segment* )             **
**     int erase_walls ()                             **
**     int place_free_space ( polygon* )              **
**     int erase_free_space ()                        **
**     int place_trajectory ( trajectory* )           **
**     int erase_trajectory ()                        **
**     int place_path ( trajectory* )                 **
**     int erase_path ()                              **
**     int place_robot_trajectory( int, trajectory* ) **
**     int erase_robot_trajectory( int )              **
**     int place_robot_path( in, trajectory* )        **
**     int erase_robot_path( int )                    **
**     int exec_trajectory ( int, trajectory* )       **
**     int move_platform ( int, angle* )              **
**     int place_goal_o ( position* )                 **
**     int place_goal_no ( position* )                **
**     int erase_goal ()                              **
**     int video_on ( int )                           **
**     int video_off ( int )                          **
**     int laser_on ( int )                           **
**     int laser_off ( int )                          **
**     int disconnect ()                              **
**                                                    **
**  All these functions return TRUE is they succeed,  **
** FALSE otherwise.                                   **
**                                                    **
**  The client.c file should be compiled and archived **
** into a user library.                               **
**                                                    **
**      (C) Je'ro^me Perret, 1992                     **
\******************************************************/
/**=============================================================
  Modifications:
      03/05/1994   S.Suzuki                  add 'select_robot_type()' 
      10/06/1994   Suzuki & Perret & Fleury  adaptation for VxWorks
      24/06/1994   S.Suzuki                  add 'getenv_server()'
      22/09/1994   S.Fleury & M. Herrb       add taskVar for VxWorks
      03/03/1995   M. Herrb                  add robot_trajectory 
                                                 and robot_path
  =============================================================**/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <rpc/rpc.h>

/*  inclcude files for VxWorks  */
/*                              */
#ifdef VXWORKS
#  include  <vxWorks.h>
#  include  <hostLib.h>
#  include  <rpcLib.h>
#  include  <errnoLib.h>
#  include  <usrLib.h>
#  include <sockLib.h>
#  include <taskVarLib.h>
/*                              */
/*    inclcude files for Unix   */
/*                              */
#else
#  include  <netdb.h>
#  include  <errno.h>
#endif


#define  GDHE_CLIENT_VAR
#include "gdhe/GDHE.h"
#include "GDHE_xdr_prot.h"


/*****                  *****/
/**    MACRO DEFINITIONS   **/
/*****                  *****/


#define MAX(a,b) (((a)>(b))?(a):(b))


/*  Generic error managing macro and others*/

#ifdef VXWORKS  /** for VxWorks **/

#define TCP_ID              6
#define MSG_ERROR(message) \
   { \
        printErrno(errnoGet()); \
         return ( FALSE );  }

extern  void  xdrstdio_create();

#else  /** for UNIX **/

#if !defined(HAVE_STRERROR)
extern int   errno;
extern char* sys_errlist[];
#define MSG_ERROR(message) \
{ \
  fprintf ( stderr, "GDHE error: " #message ); \
  fprintf ( stderr, "\n" ); \
  if ( errno ) \
      fprintf ( stderr, " (%d) %s\n", errno, sys_errlist[errno] ); \
  return ( FALSE ); \
}
#else

#define MSG_ERROR(message) \
{ \
  fprintf ( stderr, "GDHE error: " #message ); \
  fprintf ( stderr, "\n" ); \
  if ( errno ) \
      fprintf ( stderr, " (%d) %s\n", errno, strerror(errno) ); \
  return ( FALSE ); \
}
#endif
#endif


/*****                     *****/
/**    VARIABLE DEFINITIONS   **/
/*****                     *****/


/* Make these variables visible, so that the user doesn't *\
\* need to see them: the functions know them already.     */

struct GDHE_CONNECTION {
    XDR   xdr;
    FILE  *stream;
    int fd;
}  *my_gdhe_connection = NULL;


#define MY_STREAM (my_gdhe_connection->stream)
#define MY_XDR    (my_gdhe_connection->xdr)
#define MY_XDR_PTR (&MY_XDR)
#define MY_FD     (my_gdhe_connection->fd)

/*----------------------------------------------------------------------
  function name : get_connection
  original date : ??/??/1992
  modified dates: 03/05/1994  bzero -> memset,  bcopy -> memcpy
                  10/06/1994  adaptation for VxWorks
------------------------------------------------------------------------

----------------------------------------------------------------------*/
#ifndef OLD_NET
int 
get_connection(char *server_name)
{
	struct addrinfo hints, *res, *res0;
	int error;
	int s;
	const char *cause = NULL;

	/* If not already been called */
	if (my_gdhe_connection == NULL) {
		if ((my_gdhe_connection = (struct GDHE_CONNECTION *)
		     malloc(sizeof(struct GDHE_CONNECTION))) == NULL) {
			MSG_ERROR("get_connection: malloc my_gdhe_connection");
		}
	}

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	error = getaddrinfo(server_name, "3000", &hints, &res0);
	if (error) {
		MSG_ERROR("get_connection/getaddrinfo");
	}
	s = -1;
	for (res = res0; res != NULL; res = res->ai_next) {
		s = socket(res->ai_family, res->ai_socktype,
			   res->ai_protocol);
		if (s < 0) {
			cause = "socket";
			continue;
		}
		if (connect(s, res->ai_addr, res->ai_addrlen) < 0) {
			cause = "connect";
			close(s);
			s = -1;
			continue;
		}
		break;
	}
	if (s < 0) {
		MSG_ERROR(cause);
	}
	freeaddrinfo(res0);

	MY_FD = s;

	/* Get a stream on the socket */
	if ((MY_STREAM = fdopen(s, "w")) == NULL ) {
		MSG_ERROR ( "couldn't get a file pointer" );
	}
	
	/* Set buffer size to 0 for this stream */
	setbuf(MY_STREAM, NULL);

	/* Create a xdr link on the socket */
	xdrstdio_create( MY_XDR_PTR, MY_STREAM, XDR_ENCODE );

	/* Everything was allright, so just return 0 */
	return TRUE;
}

#else
int 
get_connection(char *server_name)
{
	int my_socket;
	struct sockaddr_in my_socket_address;
	int address_length;
	struct linger linger_opt;
	int reuse_opt;
#ifndef VXWORKS
	struct protoent     *proto_entry;
	struct hostent      *server_address;
#else
	int server_address;
#endif
	/* If not already been called */
	if (my_gdhe_connection == NULL) {
		
#ifdef VXWORKS
		if (rpcTaskInit() != OK) {
			MSG_ERROR("get_connection/rpcTaskInit");
		}

		if (taskVarAdd(0,(int *)&my_gdhe_connection) != OK) {
			MSG_ERROR("get_connection: taskVarAdd");
		}
#endif
		if ((my_gdhe_connection = (struct GDHE_CONNECTION *)
		     malloc(sizeof(struct GDHE_CONNECTION))) == NULL) {
			MSG_ERROR("get_connection: malloc my_gdhe_connection");
		}
	}

	/* Get an entry for tcp protocol */
	/* Open a socket for the client */

#ifdef VXWORKS
	my_socket = socket(AF_INET, SOCK_STREAM, TCP_ID);
#else
	proto_entry = getprotobyname ( "tcp" );
	if (proto_entry == NULL)  {
		MSG_ERROR("couldn't get protocol entry");
	}
	my_socket = socket(AF_INET, SOCK_STREAM, proto_entry->p_proto);
#endif
	if (my_socket == -1) {
		MSG_ERROR("couldn't open client socket");
	}
	
	/* Set socket options: do not linger and reuse address */
	linger_opt.l_onoff = 0;
	linger_opt.l_linger = 0;
	if (setsockopt (my_socket, SOL_SOCKET, SO_LINGER,
			(char *)&linger_opt, sizeof(linger_opt)) == -1) {
		MSG_ERROR("setting linger option");
	}
	reuse_opt = 1;
	if (setsockopt(my_socket, SOL_SOCKET, SO_REUSEADDR,
		       (char *)&reuse_opt, sizeof(reuse_opt)) == -1) {
		MSG_ERROR("setting reuseaddr option");
	}
    
	/* Get the absolute external address of the server processor */

#ifdef VXWORKS
	server_address = hostGetByName ( server_name );
	
	if ( server_address == ERROR ) {
		MSG_ERROR ( "couldn't get server address" );
	}

	/* Build address of socket */
	memset((char *)&my_socket_address, '\0', sizeof(my_socket_address));
	my_socket_address.sin_family = AF_INET;
	my_socket_address.sin_port = 3000;
	my_socket_address.sin_addr.s_addr = server_address;
#else
	server_address = gethostbyname ( server_name );
	
	if (server_address == NULL) {
		MSG_ERROR ( "couldn't get server address" );
	}
      
	/* Build address of socket */
	memset((char *)&my_socket_address, '\0', sizeof(my_socket_address));
	my_socket_address.sin_family = server_address->h_addrtype;
	my_socket_address.sin_port = htons(3000);
	memcpy((char *)&my_socket_address.sin_addr,
	       server_address->h_addr, server_address->h_length);
#endif
    
	/* Get the connection */
	address_length = sizeof(my_socket_address);
	if (connect(my_socket, (struct sockaddr *)&my_socket_address, 
		    address_length) == -1 ) {
		MSG_ERROR("no connection established");
	}

	MY_FD = my_socket;

	/* Get a stream on the socket */
	if ((MY_STREAM = fdopen(my_socket, "w")) == NULL) {
		MSG_ERROR("couldn't get a file pointer");
	}
	
	/* Set buffer size to 0 for this stream */
	setbuf(MY_STREAM, NULL);
	
	/* Create a xdr link on the socket */
	xdrstdio_create(MY_XDR_PTR, MY_STREAM, XDR_ENCODE);

	/* Everything was allright, so just return 0 */
	return TRUE;
}      /* get_connection() */
#endif

/* Here come the user-friendly functions. They are much *\
** alike one another, and their code is rather straight-**
** forward: initialize a packet with the argument, and  **
\* launch it across the network.                        */

int place_robot ( int which, position* posp )
{
  packet apacket;
  
  apacket.command = PLACE_ROBOT;
  apacket.which = which;
  apacket.args.pos = posp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End place_robot()  */

int erase_robot ( int which )
{
  packet apacket;
  
  apacket.command = ERASE_ROBOT;
  apacket.which = which;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   erase_robot()  */

int exec_trajectory ( int which, trajectory* trajp)
{
  packet apacket;

  apacket.command = EXEC_TRAJ;
  apacket.which = which;
  apacket.args.traj = trajp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   exec_trajectory()  */

int set_platform ( int which, angle* angp )
{
  packet apacket;
  
  apacket.command = SET_PLATFORM;
  apacket.which = which;
  apacket.args.ang = angp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}   /*  End  set_platform() */

int move_platform ( int which, angle* angp )
{
  packet apacket;
  
  apacket.command = MOVE_PLATFORM;
  apacket.which = which;
  apacket.args.ang = angp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  move_platform()  */

int place_diamond ( v_position* vposp )
{
  packet apacket;
  
  apacket.command = PLACE_DIAMOND;
  apacket.args.vpos = vposp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  place_diamond()  */

int erase_diamond ()
{
  packet apacket;
  
  apacket.command = ERASE_DIAMOND;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}   /*  End  erase_diamond()  */

int place_doors ( v_segment* vsegp )
{
  packet apacket;
  
  apacket.command = PLACE_DOORS;
  apacket.args.vseg = vsegp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*  End  place_doors()   */

int erase_doors ()
{
  packet apacket;

  apacket.command = ERASE_DOORS;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*  End  erase_doors()  */

int place_walls ( int which, int id, v_segment* vsegp )
/* which: number of the robot 
   id:    type of the wall   
   (different kind of walls for each robot - size, color)
   */
{
  packet apacket;

  apacket.command = PLACE_WALLS;
  apacket.which = which * PACKET_DIVIDER + id;
  apacket.args.vseg = vsegp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}     /*  End  place_walls()  */ 

int place_walls_rel ( int which, int id, v_segment* vsegp )
{
  packet apacket;

  apacket.command = PLACE_WALLS_REL;
  apacket.which = which * PACKET_DIVIDER + id;
  apacket.args.vseg = vsegp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}     /*  End  place_walls_rel()  */ 

int place_walls_acc ( int which, int id, v_segment* vsegp )
{
  packet apacket;

  apacket.command = PLACE_WALLS_REL;
  apacket.which = which * PACKET_DIVIDER + id;
  apacket.args.vseg = vsegp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}     /*  End  place_walls_acc()  */ 

int erase_walls ( int which, int id )
{
  packet apacket;

  apacket.command = ERASE_WALLS;
  apacket.which = which * PACKET_DIVIDER + id;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  erase_walls()   */ 

int place_free_space ( polygon* polp )
{
  packet apacket;

  apacket.command = PLACE_FREE_SPACE;
  apacket.args.pol = polp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}   /*   End  place_free_space()  */

int erase_free_space ()
{
  packet apacket;

  apacket.command = ERASE_FREE_SPACE;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}   /*   End  erase_free_space() */

int place_trajectory ( trajectory* trajp )
{
  packet apacket;

  apacket.command = PLACE_TRAJECTORY;
  apacket.args.traj = trajp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*  End  place_trajectory()  */

int erase_trajectory ()
{
  packet apacket;

  apacket.command = ERASE_TRAJECTORY;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  erase_trajectory()  */

int place_path ( trajectory* trajp )
{
  packet apacket;

  apacket.command = PLACE_PATH;
  apacket.args.traj = trajp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  place_path()  */

int erase_path ()
{
  packet apacket;

  apacket.command = ERASE_PATH;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  erase_path()   */

int place_robot_trajectory(int which, trajectory *trajp )
{
  packet apacket;

  apacket.which = which;
  apacket.command = PLACE_ROBOT_TRAJ;
  apacket.args.traj = trajp;
  return xdr_packet(MY_XDR_PTR, &apacket);
} /* place_robot_trajectory */

int erase_robot_trajectory(int which)
{
  packet apacket;

  apacket.which = which;
  apacket.command = ERASE_ROBOT_TRAJ;
  return xdr_packet(MY_XDR_PTR, &apacket);
} /* erase_robot_trajectory */

int place_robot_path(int which, trajectory *trajp )
{
  packet apacket;

  apacket.which = which;
  apacket.command = PLACE_ROBOT_PATH;
  apacket.args.traj = trajp;
  return xdr_packet(MY_XDR_PTR, &apacket);
} /* place_robot_path */

int erase_robot_path(int which)
{
  packet apacket;

  apacket.which = which;
  apacket.command = ERASE_ROBOT_PATH;
  return xdr_packet(MY_XDR_PTR, &apacket);
} /* erase_robot_trajectory */

int place_rs_trajectory(int which, int id, rs_trajectory *trajp)
{
    packet apacket;
    
    apacket.which = which * PACKET_DIVIDER + id;
    apacket.command = PLACE_RS_TRAJ;
    apacket.args.rs_traj = trajp;
    return(xdr_packet(MY_XDR_PTR, &apacket));
}

int erase_rs_trajectory(int which, int id)
{
    packet apacket;
    apacket.which = which * PACKET_DIVIDER + id;
    apacket.command = ERASE_RS_TRAJ;
    return(xdr_packet(MY_XDR_PTR, &apacket));
}

int place_goal_o ( position* posp )
{
  packet apacket;
  
  apacket.command = PLACE_GOAL_O;
  apacket.args.pos = posp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   place_goal_o()  */

int place_goal_no ( position* posp )
{
  packet apacket;
  
  apacket.command = PLACE_GOAL_NO;
  apacket.args.pos = posp;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  place_goal_no()  */

int erase_goal ()
{
  packet apacket;

  apacket.command = ERASE_GOAL;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End  erase_goal()  */

int video_on ( int which )
{
  packet apacket;

  apacket.command = VIDEO_ON;
  apacket.which = which;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   video_on()   */

int video_off ( int which )
{
  packet apacket;

  apacket.command = VIDEO_OFF;
  apacket.which = which;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   video_off()   */

int laser_on ( int which )
{
  packet apacket;

  apacket.command = LASER_ON;
  apacket.which = which;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   laser_on()   */

int laser_off ( int which )
{
  packet apacket;

  apacket.command = LASER_OFF;
  apacket.which = which;
  return xdr_packet ( MY_XDR_PTR, &apacket );
}    /*   End   laser_off()   */

int
place_dependency_edge(int from, int to)
{
    packet apacket;
    static int sto;
    
    sto = to;
    apacket.command = PLACE_DEPENDENCY_EDGE;
    apacket.which = from;
    apacket.args.edge_to = &sto;
    return xdr_packet(MY_XDR_PTR, &apacket);
}

int 
erase_dependency_edge(int from, int to)
{
    packet apacket;
    static int sto; 

    sto = to;
    apacket.command = ERASE_DEPENDENCY_EDGE;
    apacket.which = from;
    apacket.args.edge_to = &sto;
    return xdr_packet(MY_XDR_PTR, &apacket);
}    

int 
place_dependency_cycle(int which, dependency_cycle *cyclep)
{
    packet apacket;
    
    apacket.command = PLACE_DEPENDENCY_CYCLE;
    apacket.which = which;
    apacket.args.cycle = cyclep;
    return xdr_packet(MY_XDR_PTR, &apacket);
}

int 
erase_dependency_cycle(int which)
{
    packet apacket;
    
    apacket.command = ERASE_DEPENDENCY_CYCLE;
    apacket.which = which;
    return xdr_packet(MY_XDR_PTR, &apacket);
}

int
place_mission_goal(int which, char *message)
{
    packet apacket;
    
    apacket.command = PLACE_MISSION_GOAL;
    apacket.which = which;
    apacket.args.str = message;
    return xdr_packet(MY_XDR_PTR, &apacket);
}

int
erase_mission_goal(int which)
{
    packet apacket;

    apacket.command = ERASE_MISSION_GOAL;
    apacket.which = which;
    return xdr_packet(MY_XDR_PTR, &apacket);
}

int
eval_expression(char *expr)
{
    packet apacket;
    int index = 0;
    int length = strlen(expr);
    int limit = length - (EVAL_PACKET_LENGTH - 1);
    char buffer[EVAL_PACKET_LENGTH];

    apacket.which = 0;
    
    if (length > EVAL_BUFFER_LENGTH)
        return FALSE;

    if (length >= EVAL_PACKET_LENGTH)
    {
        buffer[EVAL_PACKET_LENGTH - 1] = 0;
        apacket.command = EVAL_EXPRESSION_ADD;
        apacket.args.str = buffer;

        for (index = 0; index < limit; index += EVAL_PACKET_LENGTH - 1)
        {
            strncpy(buffer, expr + index, EVAL_PACKET_LENGTH - 1);
            if (! xdr_packet(MY_XDR_PTR, &apacket))
                return FALSE;
        }
    }

    apacket.command = EVAL_EXPRESSION;
    apacket.args.str = expr + index;
    
    return xdr_packet(MY_XDR_PTR, &apacket);
}

#ifdef HAVE_GDHE_TERRAIN
int place_terrain(int which, GDHE_MNT *terrain)
{
    packet apacket;
    
    apacket.command = PLACE_TERRAIN;
    apacket.which = which;
    apacket.args.terrain = terrain;
    return xdr_packet(MY_XDR_PTR, &apacket);
}

int erase_terrain(int which)
{
    packet apacket;
    
    apacket.command = ERASE_TERRAIN;
    apacket.which = which;
    return xdr_packet(MY_XDR_PTR, &apacket);
}
#endif /* HAVE_GDHE_TERRAIN */

int disconnect ()
{
  packet apacket;
  int error;

  apacket.command = DISCONNECT;
  error = xdr_packet ( MY_XDR_PTR, &apacket );
  xdr_destroy ( MY_XDR_PTR );
  fclose ( MY_STREAM );
  close(MY_FD);
  return error;
}    /*   End  disconnect()  */

/*----------------------------------------------------------------------
  function name : select_robot_type
  original date : 03/05/1994
  modified dates: 
------------------------------------------------------------------------

----------------------------------------------------------------------*/
int  select_robot_type( int  which, int  *type)
{
  packet apacket;
  
  apacket.command = SELECT_ROBOT_TYPE;
  apacket.which = which;
  apacket.args.robot_type = type;
  return xdr_packet ( MY_XDR_PTR, &apacket );

}   /*   End  select_robot_type()  */


/*----------------------------------------------------------------------
  function name : update_per_obst
  original date : 14/10/1994
  modified dates: 
------------------------------------------------------------------------
  updates the perceived obstacles.
------------------------------------------------------------------------
----------------------------------------------------------------------*/

int update_per_obst ( int which, polygon *pols )
{
    packet apacket;

    apacket.command = UPDATE_PER_OBST;
    apacket.which = which;
    apacket.args.pol = pols;
    return xdr_packet ( MY_XDR_PTR, &apacket );
}

int erase_per_obst ( int which )
{
    packet apacket;

    apacket.command = ERASE_PER_OBST;
    apacket.which = which;
    return xdr_packet ( MY_XDR_PTR, &apacket );
}

/*----------------------------------------------------------------------
  close_all()
----------------------------------------------------------------------*/

int
close_all(void)
{ 
    packet apacket;
    
    apacket.command = CLOSE_ALL;
    return(xdr_packet(MY_XDR_PTR, &apacket));
}

/*----------------------------------------------------------------------
  function name : getenv_server();
  original date : 25/06/1994
  modified dates: 
------------------------------------------------------------------------
  get a name of the GDHE server from the environment variable.
  the variable name is defined by a macro in the file 'packets.h'
------------------------------------------------------------------------
  return : string,  if the envirenment variable is set.
           NULL,  if the env. variable dosen't exist.
----------------------------------------------------------------------*/
char  *getenv_server()
{
  char *s;

  s = (char *)getenv("GDHESERVER");

#ifndef VXWORKS
  if ( s == NULL )
    MSG_ERROR("Unset Environemnt variable 'GDHESERVER'");
#endif

  return s;
}

/*==================================================================
  Following functions are not recommended to use in the usual client.

  ==================================================================*/

/*----------------------------------------------------------------------
  function name : get_connected_xdr
  original date : 27/09/1994
  modified dates: 
------------------------------------------------------------------------

----------------------------------------------------------------------*/
XDR  get_connected_xdr()
{
  return MY_XDR;
}

/*----------------------------------------------------------------------
  function name : reset_connected_xdr()
  original date : 
  modified dates: 
------------------------------------------------------------------------

----------------------------------------------------------------------*/
void reset_connected_xdr()
{
  my_gdhe_connection = NULL;
}

